@@ -202,6 +202,8 @@ class BacnetApplication(BIPSimpleApplication):
             self.date_response = datetime.now()
 
             read_key = '%s/%u' % (str(apdu.pduSource), int(apdu.iAmDeviceIdentifier[1]))
+            if apdu.pduSource.addrType == Address.remoteStationAddr:
+                read_key = f"{read_key}/{self.nsap.router_info_cache.get_router_info(apdu.pduSource.addrNet)[1]}"
 
             if read_key not in self.bacnet_device_address_dict.keys():
                 if self.engine_callback:
@@ -428,7 +430,7 @@ class BacnetApplication(BIPSimpleApplication):
                                         property_dict[property_name] = str(property_value.cast_out(bacnet_data_type))
 
                         if str(object_identifier[0]) in self.bacnet_support_property:
-                            bacnet_point_property = {'point_name': bacnet_point_name,'index': bacnet_context.bacnet_array_index, 'point_writable': 'True', 'device_address': '%s/%s' % (str(bacnet_context.bacnet_address), str(bacnet_context.bacnet_device_id[1])),'point_type': str(object_identifier[0]), 'point_property': 'presentValue', 'point_address': str(object_identifier[1]), 'description': ''}
+                            bacnet_point_property = {'point_name': bacnet_point_name,'index': bacnet_context.bacnet_array_index, 'point_writable': 'True', 'point_device_address': '%s/%s' % (str(bacnet_context.bacnet_address), str(bacnet_context.bacnet_device_id[1])),'point_type': str(object_identifier[0]), 'point_property': 'presentValue', 'point_address': str(object_identifier[1]), 'point_description': ''}
                             bacnet_point_property.update(property_dict)
                             bacnet_context.bacnet_object_dict[bacnet_point_name] = bacnet_point_property
 
@@ -465,7 +467,13 @@ class BacnetApplication(BIPSimpleApplication):
     def get_bacnet_dest_address(self, address_key:str):
         if address_key in self.bacnet_device_address_dict.keys():
             return self.bacnet_device_address_dict[address_key].pduSource
-        return Address(str(address_key.split('/')[0]))
+
+        address = address_key.split('/')
+        if len(address) == 3:
+            self.add_route_by_device(str(address[2]), address_key)
+            return Address(str(address[0]))
+
+        return Address(str(address[0]))
 
     def _read_values(self, bacnet_point_dict: dict, bacnet_cmd_interval: float = 0.3, bacnet_multi_read: int = 20, callback_fun=None) -> dict:
         result_dict = {}
@@ -734,6 +742,11 @@ class BacnetApplication(BIPSimpleApplication):
                 self.lock_read_write_handler.release()
         return False
 
+    def add_route_by_device(self, router_address: str, device_address: str):
+        if len(router_address) > 0 and len(device_address) > 0:
+            if device_address.find(':') > 0:
+                self.add_router(router_address, int(str(device_address.split(':')[0])))
+
     # Add network routing for access across network segments(6:2)
     def add_router(self, bacnet_address: str, network: int):
         try:
@@ -743,7 +756,7 @@ class BacnetApplication(BIPSimpleApplication):
             if network not in self.address_network_dict[bacnet_address]:
                 self.address_network_dict[bacnet_address].append(network)
                 # rtn 192.168.1.184:47808 6
-                self.bacnet_application.nsap.add_router_references(None, Address(bacnet_address),self.address_network_dict[bacnet_address])
+                self.nsap.add_router_references(None, Address(bacnet_address),self.address_network_dict[bacnet_address])
         except Exception as e:
             print('bacnet add_router error(%s)' % e.__str__())
 
@@ -787,7 +800,7 @@ class BacnetReadWriteApplication(BIPSimpleApplication):
     def make_request_iocb(self, request, expected_device_id=None):
         self.expected_device_id = expected_device_id
         iocb = IOCB(request)
-        self.request_io(iocb)
+        deferred(self.request_io, iocb)
         run()
         if request.apduInvokeID is None:
             self.bacnet_error = True
@@ -810,7 +823,12 @@ class BacnetReadWriteApplication(BIPSimpleApplication):
     def get_bacnet_dest_address(self, address_key:str):
         if address_key in self.bacnet_device_address_dict.keys():
             return self.bacnet_device_address_dict[address_key].pduSource
-        return Address(str(address_key.split('/')[0]))
+        address = address_key.split('/')
+        if len(address) == 3:
+            self.add_route_by_device(str(address[2]), address_key)
+            return Address(str(address[0]))
+
+        return Address(str(address[0]))
 
     def _read_values(self, bacnet_point_dict: dict, bacnet_cmd_interval: float = 0.3, bacnet_multi_read: int = 20, callback_fun=None) -> dict:
         result_dict = {}
@@ -1046,6 +1064,11 @@ class BacnetReadWriteApplication(BIPSimpleApplication):
                 self.lock_read_write_handler.release()
         return False
 
+    def add_route_by_device(self, router_address: str, device_address: str):
+        if len(router_address) > 0 and len(device_address) > 0:
+            if device_address.find(':') > 0:
+                self.add_router(router_address, int(str(device_address.split(':')[0])))
+
     # Add network routing for access across network segments(6:2)
     def add_router(self, bacnet_address: str, network: int):
         try:
@@ -1055,7 +1078,7 @@ class BacnetReadWriteApplication(BIPSimpleApplication):
             if network not in self.address_network_dict[bacnet_address]:
                 self.address_network_dict[bacnet_address].append(network)
                 # rtn 192.168.1.184:47808 6
-                self.bacnet_application.nsap.add_router_references(None, Address(bacnet_address),self.address_network_dict[bacnet_address])
+                self.nsap.add_router_references(None, Address(bacnet_address),self.address_network_dict[bacnet_address])
         except Exception as e:
             print('bacnet add_router error(%s)' % e.__str__())
 
@@ -1091,7 +1114,7 @@ class BacnetScanApplication(BIPSimpleApplication):
     def make_request_iocb(self, request, expected_device_id=None):
         self.expected_device_id = expected_device_id
         iocb = IOCB(request)
-        self.request_io(iocb)
+        deferred(self.request_io, iocb)
         run()
         return iocb
 
@@ -1142,6 +1165,8 @@ class BacnetWhoIsIAmApplication(BIPSimpleApplication):
                 self.date_response = datetime.now()
 
                 read_key = '%s/%u' % (str(apdu.pduSource), int(apdu.iAmDeviceIdentifier[1]))
+                if apdu.pduSource.addrType == Address.remoteStationAddr:
+                    read_key = f"{read_key}/{self.nsap.router_info_cache.get_router_info(apdu.pduSource.addrNet)[1]}"
 
                 if read_key not in self.bacnet_device_address_dict.keys():
                     if self.engine_callback:
@@ -1216,6 +1241,7 @@ class BacnetDriver(BaseDriver):
     def __del__(self):
         super().__del__()
 
+    def exit(self):
         self.close()
 
     #close
@@ -1284,7 +1310,7 @@ class BacnetDriver(BaseDriver):
 
         #csv
         for point_name in self.dict_point.keys():
-            self.dict_bacnet_point[point_name] = BacnetPoint(bool(str(self.dict_point[point_name]['point_writable'])),str(self.dict_point[point_name]['point_name']),str(self.dict_point[point_name]['device_address']) , str(self.dict_point[point_name]['point_type']),str(self.dict_point[point_name]['point_property']),int(str(self.dict_point[point_name]['point_address'])),str(self.dict_point[point_name]['description']))
+            self.dict_bacnet_point[point_name] = BacnetPoint(bool(str(self.dict_point[point_name]['point_writable'])),str(self.dict_point[point_name]['point_name']),str(self.dict_point[point_name]['point_device_address']), str(self.dict_point[point_name]['point_type']),str(self.dict_point[point_name]['point_property']),int(str(self.dict_point[point_name]['point_address'])),str(self.dict_point[point_name]['point_description']))
 
     # ping server
     def ping_target(self):
@@ -1609,6 +1635,8 @@ class BacnetDriver(BaseDriver):
 
                     if read_result.propertyAccessError is None:
                         property_name = str(property_identifier)
+                        if property_name == 'description':
+                            property_name = 'point_description'
                         property_value = read_result.propertyValue
 
                         bacnet_data_type = get_datatype(object_identifier[0], property_identifier)
@@ -1625,10 +1653,10 @@ class BacnetDriver(BaseDriver):
                 if str(object_identifier[0]) in self.bacnet_support_property:
                     bacnet_point_property = {'point_name': bacnet_point_name,
                                              'index': array_index, 'point_writable': 'True',
-                                             'device_address': '%s/%s' % (str(address), str(bacnet_id)),
+                                             'point_device_address': '%s/%s' % (str(address), str(bacnet_id)),
                                              'point_type': str(object_identifier[0]),
                                              'point_property': 'presentValue',
-                                             'point_address': str(object_identifier[1]), 'description': ''}
+                                             'point_address': str(object_identifier[1]), 'point_description': ''}
                     bacnet_point_property.update(property_dict)
                     bacnet_point_dict[bacnet_point_name] = bacnet_point_property
         return bacnet_point_dict
