@@ -687,6 +687,9 @@ class ModbusDriver(BaseDriver):
     def __del__(self):
         super().__del__()
 
+    def exit(self):
+        pass
+
     def configure(self):
         # config
         self.modbus_time_out = int(str(self.dict_config.get("timeout", "5")))
@@ -698,7 +701,7 @@ class ModbusDriver(BaseDriver):
         for point_name in self.dict_point.keys():
             self.dict_modbus_point[point_name] = ModbusPoint(bool(str(self.dict_point[point_name]['point_writable'])),
                                                                  str(self.dict_point[point_name]['point_name']),
-                                                                 str(self.dict_point[point_name]['device_address']),
+                                                                 str(self.dict_point[point_name]['point_device_address']),
                                                                  int(str(self.dict_point[point_name]['point_slave_id'])),
                                                                  int(str(self.dict_point[point_name]['point_fun_code'])),
                                                                  int(str(self.dict_point[point_name]['point_address'])),int(str(self.dict_point[point_name]['point_data_type'])),int(str(self.dict_point[point_name]['point_data_length'])),float(str(self.dict_point[point_name]['point_scale'])))
