@@ -49,27 +49,17 @@ current_client = None
 
 def get_sessions(host='localhost', port=OPC_PORT):
     """Return sessions in OpenOPC Gateway Service as GUID:host hash"""
-    import Pyro4.core
     server_obj = Pyro4.Proxy("PYRO:opc@{0}:{1}".format(host, port))
     return server_obj.get_clients()
 
 def open_client(host='localhost', port=OPC_PORT):
     """Connect to the specified OpenOPC Gateway Service"""
-    import Pyro4.core
     server_obj = Pyro4.Proxy("PYRO:opc@{0}:{1}".format(host, port))
     return server_obj.create_client()
 
 @Pyro4.expose  # needed for 4.55
 class OPCClient():
 
-    class TimeoutError(Exception):
-        def __init__(self, txt):
-            Exception.__init__(self, txt)
-
-    class OPCError(Exception):
-        def __init__(self, txt):
-            Exception.__init__(self, txt)
-
     class GroupEvents:
         def __init__(self):
             self.client = current_client
@@ -139,7 +129,7 @@ class OPCClient():
                 break
             except pythoncom.com_error as err:
                 if i == len(opc_class_list) - 1:
-                    raise self.OPCError(f"Dispatch: {self._get_error_str(err)}")
+                    raise Exception(f"Dispatch: {self._get_error_str(err)}")
 
     ###base tool function
     def _exceptional(self, func, alt_return=None, alt_exceptions=(Exception,), final=None, catch=None):
@@ -195,14 +185,14 @@ class OPCClient():
                 self.opc_client.Connect(s, opc_host)
             except pythoncom.com_error as err:
                 if len(opc_server_list) == 1:
-                    raise self.OPCError(f"Connect: {s} fail({self._get_error_str(err)})")
+                    raise Exception(f"Connect: {s} fail({self._get_error_str(err)})")
 
             self.opc_client.ClientName = self.opc_client_name
             self.opc_connected = True
             break
 
     #连接
-    def connect(self, opc_server: Optional[str]=None, opc_host: str='localhost'):
+    def connect(self, opc_server: Optional[str]=None, opc_host: str='localhost') -> bool:
         """Connect to the specified OPC server"""
         if self.opc_server == opc_server and self.opc_host == opc_host and self.opc_connected:
             pass
@@ -220,18 +210,17 @@ class OPCClient():
             self._connect(opc_server_list, opc_host)
 
             if not self.opc_connected:
-                raise self.OPCError('Connect: Cannot connect to any of the servers in the OPC_SERVER list')
+                raise Exception('Connect: Cannot connect to any of the servers in the OPC_SERVER list')
 
             # With some OPC servers, the next OPC call immediately after Connect() will occationally fail.  Sleeping for 1/100 second seems to fix this.
             time.sleep(0.01)
 
             self.opc_server = opc_server
-            if opc_host == 'localhost':
-                opc_host = socket.gethostname()
             self.opc_host = opc_host
 
             # On reconnect we need to remove the old group names from OpenOPC's internal cache since they are now invalid
             self._reset()
+        return self.opc_connected
 
     def _reset(self):
         self.opc_groups = {}
@@ -248,8 +237,9 @@ class OPCClient():
 
     def _disconnect(self):
         try:
-            self.opc_client.Disconnect()
-        except self.OPCError:
+            if self.opc_client:
+                self.opc_client.Disconnect()
+        except Exception:
             pass
 
     def close(self, del_object: bool=True):
@@ -257,8 +247,8 @@ class OPCClient():
         try:
             self.remove_groups(self.groups())
         except pythoncom.com_error as err:
-            raise self.OPCError(f"Disconnect: {self._get_error_str(err)}")
-        except self.OPCError:
+            raise Exception(f"Disconnect: {self._get_error_str(err)}")
+        except Exception:
             pass
         finally:
             self.opc_connected = False
@@ -273,10 +263,11 @@ class OPCClient():
 
     def _remove_group(self, group_name: str):
         try:
-            pythoncom.CoInitialize()
-            self.opc_client.OPCGroups.Remove(group_name)
+            if self.opc_client:
+                pythoncom.CoInitialize()
+                self.opc_client.OPCGroups.Remove(group_name)
         except pythoncom.com_error as err:
-            raise self.OPCError(f"RemoveGroup: {self._get_error_str(err)}")
+            raise Exception(f"RemoveGroup: {self._get_error_str(err)}")
 
     def remove_groups(self, groups: list):
         """Remove the specified tag group(s)"""
@@ -295,7 +286,7 @@ class OPCClient():
                 if group in self.opc_group_server_handles.keys():
                     del self.opc_group_server_handles[group]
         except pythoncom.com_error as err:
-            raise self.OPCError(f'remove_groups: {self._get_error_str(err)}')
+            raise Exception(f'remove_groups: {self._get_error_str(err)}')
 
     #
     def add_group(self, group_name: str, update_rate: int = 1000) -> bool:
@@ -313,7 +304,7 @@ class OPCClient():
                 current_client = self
                 self.opc_group_hooks[group_name] = win32com.client.WithEvents(opc_group, self.GroupEvents)
             except pythoncom.com_error as err:
-                raise self.OPCError(f"add_group: {self._get_error_str(err)}")
+                raise Exception(f"add_group: {self._get_error_str(err)}")
         return True
 
     #检测有效性
@@ -328,7 +319,7 @@ class OPCClient():
             except:
                 pass
             return errors
-        raise self.OPCError(f"check_items fail")
+        raise Exception(f"check_items fail")
 
     def _add_items(self, group_name: str, client_handles: list, valid_tags: list):
         if group_name in self.opc_groups.keys():
@@ -354,8 +345,6 @@ class OPCClient():
             if not valid:
                 raise TypeError("iread(): 'tags' parameter must be a string or a list of strings")
 
-            errors = self.check_items(group_name, item_tags.copy())
-
             new_tags = []
             for item_tag in item_tags:
                 if self._check_item_exist(group_name, item_tag) is False:
@@ -368,36 +357,39 @@ class OPCClient():
             if group_name not in self.opc_group_tags.keys():
                 self.opc_group_tags[group_name] = {}
 
-            for i, item_tag in enumerate(new_tags):
-                if errors[i] == 0:
-                    valid_tags.append(item_tag)
-                    self.opc_client_handle = self.opc_client_handle + 1
-                    self.opc_group_tags[group_name][item_tag] = self.opc_client_handle
-                    client_handles.append(self.opc_client_handle)
-                else:
-                    error_msgs[item_tag] = self.opc_client.GetErrorString(errors[i])
+            if len(new_tags) > 0:
+                errors = self.check_items(group_name, new_tags.copy())
 
-            if len(valid_tags) > 0:
-                server_handles, errors = self._add_items(group_name, client_handles.copy(), valid_tags.copy())
-                if len(server_handles) > 0:
-                    valid_tags_tmp = []
-                    server_handles_tmp = []
+                for i, item_tag in enumerate(new_tags):
+                    if errors[i] == 0:
+                        valid_tags.append(item_tag)
+                        self.opc_client_handle = self.opc_client_handle + 1
+                        self.opc_group_tags[group_name][item_tag] = self.opc_client_handle
+                        client_handles.append(self.opc_client_handle)
+                    else:
+                        error_msgs[item_tag] = self.opc_client.GetErrorString(errors[i])
 
-                    if group_name not in self.opc_group_server_handles.keys():
-                        self.opc_group_server_handles[group_name] = {}
+                if len(valid_tags) > 0:
+                    server_handles, errors = self._add_items(group_name, client_handles.copy(), valid_tags.copy())
+                    if len(server_handles) > 0:
+                        valid_tags_tmp = []
+                        server_handles_tmp = []
 
-                    for i, tag in enumerate(valid_tags):
-                        if errors[i] == 0:
-                            valid_tags_tmp.append(tag)
-                            server_handles_tmp.append(server_handles[i])
-                            self.opc_group_server_handles[group_name][tag] = server_handles[i]
-                        else:
-                            error_msgs[tag] = self.opc_client.GetErrorString(errors[i])
-                    return True if len(error_msgs) == 0 else False, valid_tags, error_msgs
-                else:
-                    return False, [], {}
-            return True, [], {}
-        return False, [], {}
+                        if group_name not in self.opc_group_server_handles.keys():
+                            self.opc_group_server_handles[group_name] = {}
+
+                        for i, tag in enumerate(valid_tags):
+                            if errors[i] == 0:
+                                valid_tags_tmp.append(tag)
+                                server_handles_tmp.append(server_handles[i])
+                                self.opc_group_server_handles[group_name][tag] = server_handles[i]
+                            else:
+                                error_msgs[tag] = self.opc_client.GetErrorString(errors[i])
+                        return True if len(valid_tags_tmp) > 0 else False, valid_tags_tmp, error_msgs, ''  #只要有一个点添加成功即为正确
+                    return False, [], {}, 'add item fail'    #添加点失败
+                return False, [], error_msgs, ''    #点都不正确
+            return True, [], {}, 'no item need add'    #无需额外添加点
+        return False, [], {}, 'group not exist'    #组不存在
 
     def _remove_items(self, group_name: str, server_handles: list):
         if group_name in self.opc_groups.keys():
@@ -406,7 +398,7 @@ class OPCClient():
             try:
                 return opc_items.AddItems(len(server_handles) - 1, server_handles)
             except pythoncom.com_error as err:
-                raise self.OPCError(f"RemoveItems: {self._get_error_str(err)}")
+                raise Exception(f"RemoveItems: {self._get_error_str(err)}")
 
     def remove_items(self, group_name: str, item_tags: list):
         if group_name in self.opc_groups.keys():
@@ -468,7 +460,7 @@ class OPCClient():
                 values, errors, qualities, timestamps = opc_group.SyncRead(data_source, len(server_handles_) - 1, server_handles_)
                 self._parse_value(group_name, item_tags, values, errors, qualities, timestamps)
             except pythoncom.com_error as err:
-                raise self.OPCError(f"SyncRead: {self._get_error_str(err)}")
+                raise Exception(f"SyncRead: {self._get_error_str(err)}")
 
     def async_read(self, group_name: str, server_handles: dict):
         if group_name in self.opc_groups.keys():
@@ -483,9 +475,9 @@ class OPCClient():
                 server_handles_.insert(0, 0)
                 opc_group.AsyncRead(len(server_handles_) - 1, server_handles_, pythoncom.Missing, self.opc_transaction_id)
                 if not event.wait(self.opc_async_timeout):
-                    raise TimeoutError(f"AsyncRead: Timeout")
+                    raise Exception(f"AsyncRead: Timeout")
             except pythoncom.com_error as err:
-                raise self.OPCError(f"AsyncRead: {self._get_error_str(err)}")
+                raise Exception(f"AsyncRead: {self._get_error_str(err)}")
             finally:
                 del self.opc_group_transaction_id[self.opc_transaction_id]
 
@@ -499,9 +491,9 @@ class OPCClient():
                 self.opc_group_transaction_id[self.opc_transaction_id] = event
                 opc_group.AsyncRefresh(data_source, self.opc_transaction_id)
                 if not event.wait(self.opc_async_timeout):
-                    raise TimeoutError(f"AsyncRead: Timeout")
+                    raise Exception(f"AsyncRead: Timeout")
             except pythoncom.com_error as err:
-                raise self.OPCError(f"AsyncRefresh: {self._get_error_str(err)}")
+                raise Exception(f"AsyncRefresh: {self._get_error_str(err)}")
             finally:
                 del self.opc_group_transaction_id[self.opc_transaction_id]
 
@@ -510,7 +502,7 @@ class OPCClient():
         if group_name in self.opc_groups.keys():
             if action == '':    #订阅变化数据
                 pass
-            elif action in ['sync', 'async', 'refresh']:    #sync同步读取
+            elif action in ['sync', 'async']:    #sync同步读取
                 data_source = self.default_data_source_cache if source =='cache' else self.default_data_source_device
                 server_handles = {}
                 for item_tag in item_tags:
@@ -537,7 +529,12 @@ class OPCClient():
         results = {}
         if self.opc_connected is True:
             if len(item_tags) > 0:
-                result, success, errors = self.add_items(group_name, item_tags)
+                result, success, errors, msg = self.add_items(group_name, item_tags)
+                if result is False and len(msg) > 0:
+                    print(f"add_items error({msg}))")
+                if len(errors) > 0:
+                    print(f"add_items fail({errors})")
+
                 if len(success) > 0:
                     action = 'sync'
 
@@ -588,7 +585,7 @@ class OPCClient():
                     else:
                         results[item_tag] = False
             except pythoncom.com_error as err:
-                raise self.OPCError(f'SyncWrite: {self._get_error_str(err)}')
+                raise Exception(f'SyncWrite: {self._get_error_str(err)}')
         else:
             for item_tag in item_tags:
                 results[item_tag] = False
@@ -609,9 +606,9 @@ class OPCClient():
                 for item_tag in item_tags:
                     results[item_tag] = True
                 if not event.wait(self.opc_async_timeout):
-                    raise TimeoutError(f"AsyncRead: Timeout")
+                    raise Exception(f"AsyncRead: Timeout")
             except pythoncom.com_error as err:
-                raise self.OPCError(f'AsyncWrite: {self._get_error_str(err)}')
+                raise Exception(f'AsyncWrite: {self._get_error_str(err)}')
             finally:
                 del self.opc_group_transaction_id[self.opc_transaction_id]
         else:
@@ -642,7 +639,11 @@ class OPCClient():
         results = {}
         if self.opc_connected is True:
             if len(items_values) > 0:
-                self.add_items(group_name, list(items_values.keys()))
+                result, success, errors, msg = self.add_items(group_name, list(items_values.keys()))
+                if result is False:
+                    print(f"add_items error({msg}))")
+                if len(errors) > 0:
+                    print(f"add_items fail({errors})")
 
             valid_item_values = {}
             if len(items_values) > 0:
@@ -742,7 +743,7 @@ class OPCClient():
                 for p in tag_properties: yield p
 
         except pythoncom.com_error as err:
-            raise self.OPCError(f"properties: {self._get_error_str(err)}")
+            raise Exception(f"properties: {self._get_error_str(err)}")
 
     def properties(self, tags: list, id=None):
         """Return list of property tuples (id, name, value) for the specified tag(s) """
@@ -844,7 +845,7 @@ class OPCClient():
                             nodes[node] = True
 
         except pythoncom.com_error as err:
-            raise self.OPCError(f"list: {self._get_error_str(err)}")
+            raise Exception(f"list: {self._get_error_str(err)}")
 
     def list_items(self, paths: str = '*', recursive: bool = False, flat: bool = False, include_type: bool = False):
         """Return list of item nodes at specified path(s) (tree browser)"""
@@ -859,7 +860,7 @@ class OPCClient():
             servers = [s for s in servers if s != None]
             return servers
         except pythoncom.com_error as err:
-            raise self.OPCError(f"servers: {self._get_error_str(err)}")
+            raise Exception(f"servers: {self._get_error_str(err)}")
 
     def info(self):
         """Return list of (name, value) pairs about the OPC server"""
@@ -899,13 +900,13 @@ class OPCClient():
             return info_list
 
         except pythoncom.com_error as err:
-            raise self.OPCError(f"info: {self._get_error_str(err)}")
+            raise Exception(f"info: {self._get_error_str(err)}")
 
     def ping(self):
         """Check if we are still talking to the OPC server"""
         try:
             # Convert OPC server time to milliseconds
-            opc_serv_time = int(float(self.opc_client.CurrentTime) * 1000000.0)
+            opc_serv_time = self.opc_client.CurrentTime
             if opc_serv_time == self.opc_prev_serv_time:
                 return False
             else:
@@ -974,13 +975,14 @@ class OPCDAPoint(BasePoint):
 
 class OPCDAEngine(Thread):
 
-    def __init__(self, server: str, host: str, group: str, read_limit: int, update_rate: int, point_dict: dict, call_back_func=None):
+    def __init__(self, server: str, host: str, action: str, group: str, read_limit: int, update_rate: int, point_dict: dict, call_back_func=None):
         Thread.__init__(self)
 
         #param#########
         self.server = server
         self.host = host
         self.group = group
+        self.action = action
 
         self.read_limit = read_limit
         self.update_rate = update_rate
@@ -1013,7 +1015,7 @@ class OPCDAEngine(Thread):
     def exit(self):
         try:
             self.opcda_thread_exit = True
-            self.close_opcda_client()
+            self.close_opcda_client(self.opcda_client)
         except Exception as e:
             raise e
 
@@ -1021,33 +1023,43 @@ class OPCDAEngine(Thread):
         self.opcda_client = None
         self.add_group = False
 
-    def get_opcda_client(self):
-        try:
-            if self.opcda_client == None:
-                if len(self.host) > 0:
-                    host_info = self.host.split(':')
-                    if len(host_info) == 1:
-                        self.opcda_ip = host_info[0]
-                        self.opcda_com = True
-                    elif len(host_info) == 2:
-                        self.opcda_ip = host_info[0]
-                        self.opcda_port = int(float(host_info[1]))
-                        self.opcda_com = False
+    def get_opcda_client(self, auto_connect: bool = True):
+        if self.opcda_client == None:
+            if len(self.host) > 0:
+                host_info = self.host.split(':')
+                if len(host_info) == 1:
+                    self.opcda_ip = host_info[0]
+                    self.opcda_com = True
+                elif len(host_info) == 2:
+                    self.opcda_ip = host_info[0]
+                    self.opcda_port = int(float(host_info[1]))
+                    self.opcda_com = False
 
+                opcda_client = None
+                try:
                     if self.opcda_com is True:
-                        self.opcda_client = OPCClient()
-                        self.opcda_client.connect(self.server, self.opcda_ip)
+                        opcda_client = OPCClient()
                     else:
-                        self.opcda_client = open_client(self.opcda_ip, self.opcda_port)
-                        self.opcda_client.connect(self.server)
-        except Exception as e:
-            raise e
+                        opcda_client = open_client(self.opcda_ip, self.opcda_port)
+                except Exception as e:
+                    self.close_opcda_client(opcda_client)
+                    raise e
+                self.opcda_client = opcda_client
+
+        if self.opcda_client is not None:
+            if auto_connect is True:
+                try:
+                    self.opcda_client.connect(self.server, self.opcda_ip)
+                except Exception as e:
+                    self.close_opcda_client(self.opcda_client)
+                    raise e
         return self.opcda_client
 
-    def close_opcda_client(self):
+    def close_opcda_client(self, opcda_client):
         try:
-            if self.opcda_client:
-                self.opcda_client.close()
+            if opcda_client:
+                opcda_client.close()
+            opcda_client = None
         except Exception as e:
             print(f'ERROR: close_opcda_client({e.__str__()})')
         self.reset()
@@ -1094,13 +1106,13 @@ class OPCDAEngine(Thread):
                         point_tags[point_tag].append(point_name)
 
                     if len(point_tags) >= self.read_limit:
-                        values = self.opcda_client.read(self.group, list(point_tags.keys()))
+                        values = self.opcda_client.read(self.group, list(point_tags.keys()), self.action)
                         result_success, result_error = self.update_values(values, point_tags)
                         result_dict.update(result_success)
                         point_tags = {}
 
                 if len(point_tags) >0:
-                    values = self.opcda_client.read(self.group, list(point_tags.keys()))
+                    values = self.opcda_client.read(self.group, list(point_tags.keys()), self.action)
                     result_success, result_error = self.update_values(values, point_tags)
                     result_dict.update(result_success)
         except Exception as e:
@@ -1146,7 +1158,7 @@ class OPCDAEngine(Thread):
     def search_servers(self):
         servers = []
         try:
-            if self.get_opcda_client():
+            if self.get_opcda_client(False):
                 servers = self.opcda_client.servers()
         except Exception as e:
             raise e
@@ -1199,6 +1211,7 @@ class OPCDADriver(BaseDriver):
             self.opcda_server = str(self.dict_config.get("server", ''))
             self.opcda_host = str(self.dict_config.get("host", 'localhost'))
             self.opcda_group = str(self.dict_config.get("group", 'opcda'))
+            self.opcda_action = str(self.dict_config.get("action", ''))
             self.enable = str(self.dict_config.get("enabled", "true")).lower() == 'true'  # enable
 
             self.opcda_read_limit = self.dict_config.get('read_limit', 1000)
@@ -1209,20 +1222,22 @@ class OPCDADriver(BaseDriver):
                 self.dict_opcda_point[point_name] = OPCDAPoint(bool(str(self.dict_point[point_name]['point_writable'])), str(self.dict_point[point_name]['point_name']), str(self.dict_point[point_name]['point_tag']), str(self.dict_point[point_name]['point_type']), str(self.dict_point[point_name]['point_description']))
 
             #create_engine
-            close_delay = 0
-            if self.opcda_engine != None:
-                self.opcda_engine.exit()
-                del self.opcda_engine
-                self.opcda_engine = None
-                close_delay = 2
+            close_delay = 2
+            self.exit()
 
-            self.opcda_engine = OPCDAEngine(self.opcda_server, self.opcda_host, self.opcda_group, self.opcda_read_limit, self.opcda_update_rate, self.dict_opcda_point)
+            self.opcda_engine = OPCDAEngine(self.opcda_server, self.opcda_host, self.opcda_action, self.opcda_group, self.opcda_read_limit, self.opcda_update_rate, self.dict_opcda_point)
             self.opcda_engine.start()
 
             time.sleep(close_delay)  # delay on closed socket(This prevents a fast reconnect by the client)
         except Exception as e:
             raise e
 
+    def exit(self):
+        if self.opcda_engine != None:
+            self.opcda_engine.exit()
+            del self.opcda_engine
+            self.opcda_engine = None
+
     # ping
     def ping_target(self):
         try:
