@@ -623,6 +623,9 @@ class SiemensS7Driver(BaseDriver):
     def __del__(self):
         super().__del__()
 
+    def exit(self):
+        pass
+
     def configure(self):
         try:
             # config
