@@ -152,7 +152,7 @@ class FtpAccessor:
     def download_file(self, client: FTP, remote_file_name: str, local_file_path: str, remote_file_folder: str=''):
         self.cwd_folder(client, remote_file_folder)
         result = client.retrbinary(f"RETR {remote_file_name}", open(local_file_path, 'wb').write, self.config.BUFFER_SIZE)
-        if result.find('226 Transfer complete') >= 0:
+        if result.startswith('226'):
             return True
         raise Exception(f"download fail({result})")
 
@@ -160,7 +160,7 @@ class FtpAccessor:
     def upload_file(self, client: FTP, remote_file_name: str, local_file_path: str, remote_file_folder: str=''):
         self.cwd_folder(client, remote_file_folder)
         result = client.storbinary(f"STOR {remote_file_name}", open(local_file_path, 'rb'), self.config.BUFFER_SIZE)
-        if result.find('226 Transfer complete') >= 0:
+        if result.startswith('226'):
             return True
         raise Exception(f"upload fail({result})")
 
