@@ -1,5 +1,5 @@
 import json
-from basic.encrypt.rsa import *
+from robertcommon.basic.encrypt.rsa import *
 
 def test_rsa():
     secret = 'robert'
